# ----------------------------------------------------------------------------
# Exec commands (loosely based on xtem_texmenu)
# ----------------------------------------------------------------------------

# Check whether the specified command is executable

proc which {command} {
  global env
  if [file executable $command] {return 1}
  foreach dir [split $env(PATH) :] {
    set path ${dir}/${command}
    if {[file isfile $path]&&[file executable $path]} {return 1}
  }
  return 0
}


# ----------------------------------------------------------------------------
# Executed when the child writes to stdout or stderr

proc cmd_out {} {
  global f_mkCmd p_mkCmd end_mkCmd first_line new_logline

  if {[catch {eof $f_mkCmd}]} {
    set p_mkCmd 1
    set end_mkCmd "end"
    return
  }
  if {[eof $f_mkCmd]} {
    set p_mkCmd 1
    set end_mkCmd "end"
    catch "close $f_mkCmd"
    return
  }
  set c [read $f_mkCmd]
  if {$c==""} return
  if $first_line {set new_logline 1}
  set first_line 0
  log_event $c
}


# ----------------------------------------------------------------------------
# Start ONE external command

proc exec_cmd {cmd args {pid_var "cmd_pid"} {wait ""}} {
  global $pid_var argv0 end_mkCmd f_mkCmd p_mkCmd first_line

  if ![which $cmd] {
    log_entry "ERROR: command '$cmd' not found"
    return 0
  }
  flush stdout
  flush stderr
  set end_mkCmd "start"

#.catch command output to display it into the logbox
  set first_line 1
  set f_mkCmd [open "| $cmd $args |& cat" RDWR]
  fileevent $f_mkCmd readable "cmd_out"
  fconfigure $f_mkCmd -blocking 0 -buffering none
  set p_mkCmd [pid $f_mkCmd]
  set $pid_var [lindex $p_mkCmd 0]

#.wait for command termination if requested
  if {$wait != ""} {tkwait variable end_mkCmd}

#.restore original default bindings
  if [string compare [file tail $argv0] "palfax"] {
    std_keys
    act_keys 1
  }
  return 1
}

